@@ -1,8 +1,8 @@
 #' Returns ECB data for a given flow and key
 #'
-#' @param flow `character(1)` flow to query.
-#' @param key `character(1)` key to query.
-#' @param start_period `character(1)` start date of the data. Supported formats:
+#' @param flow (`character(1)`) flow to query.
+#' @param key (`character(1)`) key to query.
+#' @param start_period (`character(1)`) start date of the data. Supported formats:
 #'   - YYYY for annual data (e.g., "2019")
 #'   - YYYY-S\[1-2\] for semi-annual data (e.g., "2019-S1")
 #'   - YYYY-Q\[1-4\] for quarterly data (e.g., "2019-Q1")
@@ -11,17 +11,17 @@
 #'   - YYYY-MM-DD for daily and business data (e.g., "2019-01-01")
 #'   If `NULL`, no start date restriction is applied (data retrieved from the
 #'   earliest available date). Default `NULL`.
-#' @param end_period `character(1)` end date of the data, in the same format as
+#' @param end_period (`character(1)`) end date of the data, in the same format as
 #'   start_period. If `NULL`, no end date restriction is applied (data
 #'   retrieved up to the most recent available date). Default `NULL`.
-#' @param first_n `numeric(1)` number of observations to retrieve from the
+#' @param first_n (`numeric(1)`) number of observations to retrieve from the
 #'   start of the series. If `NULL`, no restriction is applied. Default `NULL`.
-#' @param last_n `numeric(1)` number of observations to retrieve from the end
+#' @param last_n (`numeric(1)`) number of observations to retrieve from the end
 #'  of the series. If `NULL`, no restriction is applied. Default `NULL`.
 #' @source <https://data.ecb.europa.eu/help/api/data>
 #' @family data
 #' @export
-#' @examples
+#' @examplesIf curl::has_internet()
 #' \donttest{
 #' # fetch US dollar/Euro exchange rate
 #' ecb_data("EXR", "D.USD.EUR.SP00.A")
@@ -43,7 +43,7 @@ ecb_data <- function(flow,
     is_count_or_null(last_n)
   )
 
-  key <- if (!is.null(key)) paste(key, collapse = "+") else "all"
+  key <- if (!is.null(key)) paste0(key, collapse = "+") else "all"
   resource <- paste("data", flow, key, sep = "/")
   body <- ecb(
     resource = resource,
@@ -52,8 +52,8 @@ ecb_data <- function(flow,
     firstNObservations = first_n,
     lastNObservations = last_n
   )
-  res <- parse_ecb_data(body)
-  as_tibble(res)
+  dt <- parse_ecb_data(body)
+  dt
 }
 
 parse_ecb_data <- function(body) {
@@ -67,7 +67,7 @@ parse_ecb_data <- function(body) {
       tolower()
     series_key <- series_key |>
       xml2::xml_attr("value") |>
-      stats::setNames(nms) |>
+      setNames(nms) |>
       as.list()
 
     attrs <- x |>
@@ -79,11 +79,11 @@ parse_ecb_data <- function(body) {
     nms <- replace(nms, nms == "title_compl", "description")
     attrs <- attrs |>
       xml2::xml_attr("value") |>
-      stats::setNames(nms) |>
+      setNames(nms) |>
       as.list()
 
     data <- c(series_key, attrs)
-    data$key <- paste(series_key, collapse = ".")
+    data$key <- paste0(series_key, collapse = ".")
 
     data$freq <- switch(data$freq,
       A = "annual",
@@ -105,14 +105,13 @@ parse_ecb_data <- function(body) {
       xml2::xml_attr("value") |>
       as.numeric()
 
-    as.data.frame(data)
+    as.data.table(data)
   })
   nms <- lapply(res, names)
   nms <- Reduce(intersect, nms)
   nms <- union(c("date", "key", "value", "title", "description"), nms)
-  res <- lapply(res, \(x) x[nms])
-  res <- do.call(rbind, res)
-  res
+  res <- lapply(res, \(x) x[, nms, with = FALSE])
+  rbindlist(res)
 }
 
 #' Returns the available ECB metadata
@@ -120,11 +119,11 @@ parse_ecb_data <- function(body) {
 #' Retrieval of the metadata stored in the ECB's time series database.
 #' Access via the SDMX Web Service API of the ECB
 #'
-#' @param type `character(1)` the type of metadata to query. One of:
+#' @param type (`character(1)`) the type of metadata to query. One of:
 #' `"datastructure"`, `"dataflow"`, `"codelist"`, or `"concept"`.
-#' @param agency `character(1)` id of the agency to query. Default `NULL`.
-#' @param id `character(1)` id of the resource to query. Default `NULL`.
-#' @returns A `data.frame()` with the queried metadata.
+#' @param agency (`character(1)`) id of the agency to query. Default `NULL`.
+#' @param id (`character(1)`) id of the resource to query. Default `NULL`.
+#' @returns A [data.table::data.table()] with the queried metadata.
 #' The columns are:
 #'   \item{agency}{The agency of the metadata}
 #'   \item{id}{The id of the metadata}
@@ -132,7 +131,7 @@ parse_ecb_data <- function(body) {
 #' @source <https://data.ecb.europa.eu/help/api/metadata>
 #' @family metadata
 #' @export
-#' @examples
+#' @examplesIf curl::has_internet()
 #' \donttest{
 #' ecb_metadata("datastructure")
 #' ecb_metadata("datastructure", "ECB")
@@ -148,21 +147,18 @@ ecb_metadata <- function(type, agency = NULL, id = NULL) {
     concept = list("conceptscheme", "//str:ConceptScheme")
   )
   res <- do.call(fetch_ecb_metadata, c(args, list(agency, id)))
-  as_tibble(res)
+  res
 }
 
 fetch_ecb_metadata <- function(resource, xpath, agency = NULL, id = NULL) {
-  stopifnot(
-    is_string_or_null(agency),
-    is_string_or_null(id)
-  )
+  stopifnot(is_string_or_null(agency), is_string_or_null(id))
   agency <- if (!is.null(agency)) toupper(agency) else "all"
   id <- if (!is.null(id)) toupper(id) else "all"
   resource <- paste(resource, agency, id, sep = "/")
   body <- ecb(resource)
   entries <- xml2::xml_find_all(body, xpath)
-  res <- parse_ecb_metadata(entries)
-  res
+  dt <- parse_ecb_metadata(entries)
+  dt
 }
 
 parse_ecb_metadata <- function(x, lang = "en") {
@@ -172,13 +168,13 @@ parse_ecb_metadata <- function(x, lang = "en") {
     nms <- node |>
       xml2::xml_find_all(sprintf(".//com:Name[@xml:lang='%s']", lang)) |>
       xml2::xml_text()
-    data.frame(agency = agency, id = id, name = nms, check.names = FALSE)
+    data.table(agency = agency, id = id, name = nms)
   })
-  do.call(rbind, res)
+  rbindlist(res)
 }
 
 ecb_error_body <- function(resp) {
-  message <- resp_body_string(resp)
+  message <- resp_body_string(resp, "UTF-8")
   docs <- "See docs at <https://data.ecb.europa.eu/help/api/status-codes>"
   c(message, docs)
 }
